import { Page, LocaleConfig, PluginFunction } from 'vuepress/core';
import { ExactLocaleConfig } from '@vuepress/helper';
import { SearchIndex } from 'slimsearch';

declare const enum IndexField {
    heading = "h",
    anchor = "a",
    text = "t",
    customFields = "c"
}
type PageIndexId = `${number}`;
interface PageIndexItem {
    id: PageIndexId;
    /** Heading */ h: string;
    /** Text */ t?: string[];
}
type SectionIndexId = `${PageIndexId}#${string}`;
interface SectionIndexItem {
    id: SectionIndexId;
    /** Heading */ h: string;
    /** Text */ t?: string[];
}
type CustomFieldIndexID = `${PageIndexId}@${number}`;
interface CustomFieldIndexItem {
    id: string;
    /** CustomFields */ c: string[];
}
type IndexItem = PageIndexItem | SectionIndexItem | CustomFieldIndexItem;
type LocaleIndex = Record<string, IndexItem[]>;
type SearchIndexStore = Record<string, SearchIndex<string, IndexItem, IndexItem>>;

type SearchProCustomFieldFormatter = string | Record<string, string>;

interface SearchProHotKeyOptions {
    /**
     * Value of `event.key` to trigger the hot key
     *
     * 热键的 `event.key` 值
     */
    key: string;
    /**
     * Whether to press `event.altKey` at the same time
     *
     * 是否同时按下 `event.altKey`
     *
     * @default false
     */
    alt?: boolean;
    /**
     * Whether to press `event.ctrlKey` at the same time
     *
     * 是否同时按下 `event.ctrlKey`
     *
     * @default false
     */
    ctrl?: boolean;
    /**
     * Whether to press `event.shiftKey` at the same time
     *
     * 是否同时按下 `event.shiftKey`
     *
     * @default false
     */
    shift?: boolean;
    /**
     * Whether to press `event.metaKey` at the same time
     *
     * 是否同时按下 `event.metaKey`
     *
     * @default false
     */
    meta?: boolean;
}

/**
 * Multi language config for `vuepress-plugin-search-pro` plugin
 *
 * `vuepress-plugin-search-pro` 插件的多语言配置
 */
interface SearchProLocaleData {
    /**
     * Search box placeholder
     *
     * 搜索框占位符文字
     */
    placeholder: string;
    /**
     * Search text
     *
     * 搜素文字
     */
    search: string;
    /**
     * Searching text
     *
     * 搜素中文字
     */
    searching: string;
    /**
     * Cancel text
     *
     * 取消文字
     */
    cancel: string;
    /**
     * Default title
     *
     * 默认标题
     */
    defaultTitle: string;
    /**
     * Select hint
     *
     * 选择提示
     */
    select: string;
    /**
     * Choose hint
     *
     * 选择提示
     */
    navigate: string;
    /**
     * Autocomplete hint
     *
     * 自动补全提示
     */
    autocomplete: string;
    /**
     * Close hint
     *
     * 关闭提示
     */
    exit: string;
    /**
     * Loading hint
     *
     * 加载提示
     */
    loading: string;
    /**
     * Search query history title
     *
     * 搜索文字历史 标题
     */
    queryHistory: string;
    /**
     * Search result history title
     *
     * 搜索结果历史 标题
     */
    resultHistory: string;
    /**
     * Search history empty hint
     *
     * 无搜索历史提示
     */
    emptyHistory: string;
    /**
     * Empty hint
     *
     * 无结果提示
     */
    emptyResult: string;
}
type SearchProLocaleConfig = ExactLocaleConfig<SearchProLocaleData>;

interface SearchProIndexOptions {
    /**
     * Function to tokenize the index field item.
     *
     * 用于对索引字段项进行分词的函数。
     */
    tokenize?: (text: string, fieldName?: string) => string[];
    /**
     * Function to process or normalize terms in the index field.
     *
     * 用于处理或规范索引字段中的术语的函数。
     */
    processTerm?: (term: string) => string | string[] | null | undefined | false;
}
interface SearchProCustomFieldOptions {
    /**
     * Custom field getter
     *
     * 自定义项目的获取器
     */
    getter: <ExtraPageData extends Record<string, unknown> = Record<never, never>, ExtraPageFrontmatter extends Record<string, unknown> = Record<string, unknown>, ExtraPageFields extends Record<string, unknown> = Record<never, never>>(page: Page<ExtraPageData, ExtraPageFrontmatter, ExtraPageFields>) => string[] | string | null | undefined;
    /**
     * Display content
     *
     * @description `$content` will be replaced by the content returned by `getter`
     *
     * 展示的内容
     *
     * @description `$content` 会被 `getter` 返回的内容替换
     *
     * @default `$content`
     */
    formatter?: SearchProCustomFieldFormatter;
}
interface DeprecatedSearchProOptions {
    /**
     * @deprecated use `indexContent` instead
     */
    fullIndex?: never;
    /**
     * @deprecated use `resultHistoryCount` instead
     */
    historyCount?: never;
    /**
     * @deprecated use `searchDelay` instead
     */
    delay?: never;
}
interface SearchProOptions extends DeprecatedSearchProOptions {
    /**
     * Whether index page content
     *
     * @description By default only headings and excerpt of the page will be indexed, and the content of the page will not be indexed. If you need to index the content of the page, you can set this option to `true`
     *
     * 是否索引正文内容
     *
     * @description 默认情况下，只会索引页面的标题和摘要，不会索引页面的正文内容。如果需要索引页面的正文内容，可以将该选项设置为 `true`
     *
     * @default false
     */
    indexContent?: boolean;
    /**
     * Whether provide auto suggestions while typing
     *
     * 是否在输入时提供自动建议
     *
     * @default true
     */
    autoSuggestions?: boolean;
    /**
     * Max stored query history count
     *
     * @description You can set it to `0` to disable it
     *
     * 存储查询历史的最大数量
     *
     * @description 可以将其设置为 `0` 来禁用
     *
     * @default 5
     */
    queryHistoryCount?: number;
    /**
     * Max stored matched result history count
     *
     * @description You can set it to `0` to disable it
     *
     * 存储结果历史的最大数量
     *
     * @description 可以将其设置为 `0` 来禁用
     *
     * @default 5
     */
    resultHistoryCount?: number;
    /**
     * Delay to start searching after input
     *
     * 结束输入到开始搜索的延时
     *
     * @default 150
     */
    searchDelay?: number;
    suggestDelay?: number;
    /**
     * Custom field for search
     */
    customFields?: SearchProCustomFieldOptions[];
    /**
     * Specify the [event.key](http://keycode.info/) of the hotkeys
     *
     * @description When hotkeys are pressed, the search box input will be focused. Set to an empty array to disable hotkeys
     *
     * 指定热键的 [event.key](http://keycode.info/)
     *
     * @description 当热键被按下时，搜索框的输入框会被聚焦，设置为空数组以禁用热键
     *
     * @default [
     *   { key: "k", ctrl: true },
     *   { key: "/", ctrl: true },
     *  ]
     */
    hotKeys?: SearchProHotKeyOptions[];
    /**
     * Output worker filename
     *
     * Worker 输出文件名
     *
     * @default "search-pro.worker.js"
     */
    worker?: string;
    /**
     * Whether enable hmr
     *
     * 是否启用 hmr
     *
     * @default false
     */
    hotReload?: boolean;
    /**
     * Locales config
     *
     * @see [default config](https://github.com/vuepress-theme-hope/vuepress-theme-hope/blob/main/packages/search-pro/src/node/locales.ts)
     *
     * 多语言选项
     *
     * @see [默认配置](https://github.com/vuepress-theme-hope/vuepress-theme-hope/blob/main/packages/search-pro/src/node/locales.ts)
     */
    locales?: LocaleConfig<SearchProLocaleData>;
    /**
     * Result Sort strategy
     *
     * @description When there are multiple matched results, the result will be sorted by the strategy. `max` means that page having higher total score will be placed in front. `total` means that page having higher max score will be placed in front.
     *
     * 结果排序策略
     *
     * @description 当有多个匹配的结果时，会按照策略对结果进行排序。`max` 表示最高分更高的页面会排在前面。`total` 表示总分更高的页面会排在前面
     *
     * @default "max"
     */
    sortStrategy?: "max" | "total";
    /**
     * Create Index option
     *
     * 创建索引选项
     */
    indexOptions?: SearchProIndexOptions;
    /**
     * Create Index option per locale
     *
     * 按语言的创建索引选项
     */
    indexLocaleOptions?: Record<string, SearchProIndexOptions>;
    /**
     * Filter pages to be indexed
     *
     * 过滤需要索引的页面
     *
     * @param page Page
     * @returns whether the page should be indexed
     */
    filter?: (page: Page) => boolean;
}

declare const searchProPlugin: (options: SearchProOptions, legacy?: boolean) => PluginFunction;

export { type CustomFieldIndexID, type CustomFieldIndexItem, type DeprecatedSearchProOptions, IndexField, type IndexItem, type LocaleIndex, type PageIndexId, type PageIndexItem, type SearchIndexStore, type SearchProCustomFieldFormatter, type SearchProCustomFieldOptions, type SearchProHotKeyOptions, type SearchProIndexOptions, type SearchProLocaleConfig, type SearchProLocaleData, type SearchProOptions, type SectionIndexId, type SectionIndexItem, searchProPlugin };
